# -*- coding: utf-8 -*-
#################################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 Julius Network Solutions SARL <contact@julius.fr>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#################################################################################

from osv import fields, osv
import datetime

def total_duration(self, cr, uid, ids, name, args, context=None, monthly=0, is_employee = False):
    pool = pooler.get_pool(cr.dbname)
    res={}
    
    for contract in self.browse(cr,uid,ids, context=context):
        total = 0
        shared = 0
        presence = 0
        lines=[]
        #For each contract, the usual_hour lines are added to the list if th contract is a monthly contract
        #and is active. The check could be done once for the contract and all the lines added to the list. 
        for line in contract.usual_hours :
            if line.contract_id.date_start <= datetime.datetime.today().strftime('%Y-%m-%d') \
            and line.contract_id.date_end >= datetime.datetime.today().strftime('%Y-%m-%d') \
            and (line.contract_id.monthly or not monthly):
                lines.append(line)
        #It seems that this instruction could just be replaced by shared_rate = contract.contract_linked_rate
        if lines :
            shared_rate = lines[0].contract_id.contract_linked_rate
        else :
            shared_rate = 0

        if is_employee :
        #There may be an error in this line
            criterias = [('employee_id','=',contract.id)]
        else :      
            if contract.employee_id :
                criterias = [('employee_id','=',contract.employee_id.id)]
            else :
                criterias = []
        #Research of all the usual_hours of the employee
        olines_id = pool.get('partner.usualhours').search(cr,uid, criterias)
        olines = pool.get('partner.usualhours').browse(cr,uid,olines_id)
        #The program filters all the found usual_hours that are not from an active contract.
        #These usual hours will be used to compute shared hours.
        slines=[]
        for line in olines :
            if line.contract_id.date_start <= datetime.datetime.today().strftime('%Y-%m-%d') \
            and  line.contract_id.date_end >=datetime.datetime.today().strftime('%Y-%m-%d') \
            and line.contract_id.id != contract.id :
                slines.append(line)
        
        for line in lines :
            sha=False
            total+= line.hour_to - line.hour_from
            for sline in slines :
                if sline.id != line.id and sline.hour_from < line.hour_to and line.hour_from < sline.hour_to\
                and sline.day_of_week==line.day_of_week :
                    shared += min(line.hour_to,sline.hour_to)-max(line.hour_from,sline.hour_from)
                    sha=True
            if not sha :
                presence += line.responsible_hours
        #The hours are balanced with the shared hours and the responsible hours rates.
        res[contract.id]=total-(1-float(shared_rate)/100)* shared - 1.0/3 * presence
        return res

#this class is employee contract
class hr_contract(osv.osv):

    #This function computes the total worked hours of an employee with the
    #usual worked hours linked with the employee.
    def _week_total(self, cr, uid, contract_id):
        contract = self.browse(cr,uid,contract_id)
        pool = pooler.get_pool(cr.dbname)
        usual_hour_ids = pool.get('partner.usualhours').search(cr, uid, [('employee_contract_id','=',contract_id)])
        usual_hours = pool.get('partner.usualhours').browse(cr,uid,usual_hour_ids)
        partner_contract_id_list = []
        
        #Research of all partner contracts linked with this employee contract
        for usual_hour in usual_hours :
            if usual_hour.contract_id.date_start <= datetime.datetime.today().strftime('%Y-%m-%d') \
            and usual_hour.contract_id.date_end >= datetime.datetime.today().strftime('%Y-%m-%d') \
            and partner_contract_id_list.count(usual_hour.contract_id.id)== 0 \
            and usual_hour.contract_id.monthly :
                partner_contract_id_list.append(usual_hour.contract_id.id)
        
        
        #Research of linked contract and shared rate
        shared_rate = 100
        for partner_contract in pool.get('partner.contract').browse(cr,uid,partner_contract_id_list) :
            if partner_contract.contract_linked :
                if partner_contract_id_list.count(partner_contract.contract_linked.id ) == 0 \
                and partner_contract.contract_linked.date_start <= datetime.datetime.today().strftime('%Y-%m-%d') \
                and partner_contract.contract_linked.date_end >=datetime.datetime.today().strftime('%Y-%m-%d') :
                    partner_contract_id_list.append(partner_contract.contract_linked.id)
                #There is a shared rate only in "mandataire" ? Should be yes, but better to check.
                if partner_contract.type == 'mandataire' :
                    shared_rate=partner_contract.contract_linked_rate
        
        #Research of all the usualhours of the employee for the partner contracts linked with the present 
        #employee contract
        all_usual_hours_id = pool.get('partner.usualhours').search(cr, uid, [('employee_id','=',contract.employee_id.id),\
                                                                            ('contract_id','in',partner_contract_id_list)])
        usual_hours = pool.get('partner.usualhours').browse(cr,uid,all_usual_hours_id)
        total = 0
        shared = 0
        presence = 0 
        susual_hours = usual_hours       
        for usual_hour in usual_hours :
            sha=False
            total+= usual_hour.hour_to - usual_hour.hour_from
            for susual_hour in susual_hours :
                if susual_hour.id != usual_hour.id and susual_hour.hour_from < usual_hour.hour_to \
                and usual_hour.hour_from < susual_hour.hour_to\
                and susual_hour.day_of_week==usual_hour.day_of_week :
                    shared += min(usual_hour.hour_to,susual_hour.hour_to)-max(usual_hour.hour_from,susual_hour.hour_from)
                    sha=True
            #If there is no shared hours, responsible presence hours are balanced
            if not sha :
                presence += usual_hour.responsible_hours
            
        total = total-(0.5)* shared - 1.0/3 * presence
        return{'shared' : 0.5 * shared,
               'presence' : presence,
               'total' : total,
               'shared_rate' : shared_rate,
               }
    
    #This function computes the total worked hours, but only for the current employee contract and for
    #monthly partner contracts. Because, monthly package depends only on monthly contracts.
    def _contract_total(self, cr, uid, contract_id):
        contract = self.browse(cr,uid,contract_id)
        pool = pooler.get_pool(cr.dbname)
        usual_hour_ids = pool.get('partner.usualhours').search(cr, uid, [('employee_contract_id','=',contract_id)])
        usual_hours = pool.get('partner.usualhours').browse(cr,uid,usual_hour_ids)
        shared_rate = 100
        for usual_hour in usual_hours :
            if usual_hour.contract_id.date_start <= datetime.datetime.today().strftime('%Y-%m-%d') \
            and usual_hour.contract_id.date_end >=datetime.datetime.today().strftime('%Y-%m-%d') \
            and usual_hour.contract_id.contract_linked \
            and usual_hour.contract_id.monthly :
                shared_rate = usual_hour.contract_id.contract_linked_rate
                
        total = 0
        presence = 0 
        susual_hours = usual_hours       
        for usual_hour in usual_hours :
            total+= usual_hour.hour_to - usual_hour.hour_from
            presence += usual_hour.responsible_hours
            
        total = total - 1.0/3 * presence
        return{'presence' : presence,
               'total' : total,
               'shared_rate' : shared_rate,
               }        


    def _total_duration(self, cr, uid, ids, name, args, context=None):
        return total_duration(self, cr, uid, ids, name, args, context=None)
        
        
    
    def _total_duration_monthly(self, cr, uid, ids, name, args, context=None):
        contract = self.browse(cr,uid,ids[0], context=context)
        
        #The week hour total is computed to know if the employee makes overtime.
        week_total = self._week_total(cr, uid, contract.id)
        tot_usual_hours_monthly = self._contract_total(cr, uid, contract.id)
        
        #This hours are only the present contract hours.
        contract_hours = tot_usual_hours_monthly['total']
        shared_rate = tot_usual_hours_monthly['shared_rate']
        
        #This level is overtime hours level, for example 35 hours a week in French legislation.
        week_hours = week_total['total']
        week_shared = week_total['shared']
        
        level = contract.salary_type.hours_a_week
        
        #Worked hours are separated between normal hours (under overtime hours),
        #first overtime area (level -> level + 8 hours)
        #and second overtime (level + 8 -> 50 hours).
        normal_hours = min(level, week_hours)
        sup1 = min(8, max(week_hours- level,0))
        sup2 = min(50 - level - 8, max(week_hours - level - 8, 0 ))
        if week_hours :
            rate = float(contract_hours)/week_hours
        else :
            rate = 0
        
        #Hours are balanced with shared hours.
        normal_hours = normal_hours * rate
        sup1 = sup1 * rate
        sup2 = sup2 * rate

        normal_hours_shared = min(level, week_shared)
        sup1_shared = min(8, max(week_shared- level,0))
        sup2_shared = min(50 - level - 8, max(week_shared - level - 8, 0 ))
        
        normal_hours -= (1 - float(shared_rate)/100) * normal_hours_shared
        sup1 -= (1 - float(shared_rate)/100) * sup1_shared
        sup2 -= (1 - float(shared_rate)/100) * sup2_shared

        #The function computes the monthly hour package. This package is computed
        #multiplying the week hours by the number of worked weeks in a year to know how
        #many hours are worked in the year. After, this result is divided by 12 to have a
        #monthly time. 
        month_hour = float(normal_hours * contract.worked_weeks)/12
        month_hour_sup1 = float(sup1 * contract.worked_weeks)/12
        month_hour_sup2 = float(sup2 * contract.worked_weeks)/12
        
        sup1_rate = contract.salary_type.sup1_rate
        sup2_rate = contract.salary_type.sup2_rate
        
        #Overtime hours are more paid.
        month_wage = month_hour * contract.gross_wage_rate
        month_wage_sup1 = month_hour_sup1 * contract.gross_wage_rate * sup1_rate
        month_wage_sup2 = month_hour_sup2 * contract.gross_wage_rate * sup2_rate
        
        res={}
        res[contract.id]={'tot_usual_hours_monthly' : normal_hours,
                          'month_hours' : month_hour,
                          'month_wage' : month_wage,
                          'tot_usual_hours_monthly_sup1' : sup1,
                          'month_hours_sup1' : month_hour_sup1,
                          'month_wage_sup1' : month_wage_sup1,
                          'tot_usual_hours_monthly_sup2' : sup2,
                          'month_hours_sup2' : month_hour_sup2,
                          'month_wage_sup2' : month_wage_sup2,
                          }
        return res


    _inherit = "hr.contract"
    _columns = {
        'tot_usual_hours' : fields.function(_total_duration, type = 'float', store = True, method=True, string = u'nombre d\'heures théoriques'),
        'tot_usual_hours_monthly' : fields.function(_total_duration_monthly, type = 'float', store = True, method=True, string = u'nombre d\'heures théoriques contrats mensuels', multi='all'),
        'month_hours' : fields.function(_total_duration_monthly, type = 'float', store = True, method=True, string = u'forfait mensuel en heures', multi='all'),
        'month_wage' : fields.function(_total_duration_monthly, type = 'float', store = True, method=True, string = u'forfait mensuel en euro', multi='all'),
        'tot_usual_hours_monthly_sup1' : fields.function(_total_duration_monthly, type = 'float', store = True, method=True, string = u'nombre d\'heures supplémentaires tranche 1 théoriques', multi='all'),
        'month_hours_sup1' : fields.function(_total_duration_monthly, type = 'float', store = True, method=True, string = u'forfait mensuel en heures supplémentaires tranche 1', multi='all'),
        'month_wage_sup1' : fields.function(_total_duration_monthly, type = 'float', store = True, method=True, string = u'forfait mensuel en euro pour supplémentaires tranche 1', multi='all'),
        'tot_usual_hours_monthly_sup2' : fields.function(_total_duration_monthly, type = 'float', store = True, method=True, string = u'nombre d\'heures supplémentaires tranche 2 théoriques contrats mensuels', multi='all'),
        'month_hours_sup2' : fields.function(_total_duration_monthly, type = 'float', store = True, method=True, string = u'forfait mensuel en heures supplémentaires tranche 2', multi='all'),
        'month_wage_sup2' : fields.function(_total_duration_monthly, type = 'float', store = True, method=True, string = u'forfait mensuel en euro pour supplémentaires tranche 2', multi='all'),
    }

hr_contract()

#class hr_employee(osv.osv):
#
#    def _total_duration(self, cr, uid, ids, name, args, context=None):
#        return total_duration(self, cr, uid, ids, name, args, context=None, is_employee = True)
#    
#        
#    _name = "hr.employee"
#    _inherit = "hr.employee"
#    _columns = {'usual_hours': fields.one2many('partner.usualhours','employee_id',u'Heures habituellement pratiquées'),
#                'tot_usual_hours' : fields.function(_total_duration, type = 'float', store = True, method=True, string = u'nombre d\'heures théoriques'),
#                }
#hr_employee()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:heures effectives / semaine x Nbre semaines travaillees / an / 12
