# -*- coding: utf-8 -*-
#################################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 Julius Network Solutions SARL <contact@julius.fr>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#################################################################################

import datetime
from osv import fields, osv
from tools.translate import _

class partner_contract(osv.osv):
    _inherit = "contract.contract"
    _description = "Parnter Contract"
    
    """This function computes the month prices and worked hours for a contract"""
    def _package_compute(self, cr, uid, ids, name, args, context=None):
        res={}
        for contract in self.browse(cr,uid,ids, context=context):
            
            full_hours_worked=0
            duration = 0
            shared = 0
            presence = 0
            
            """If the contract is linked with another, the program takes the linked contract
            usual hours to compute shared hours."""
            if contract.contract_linked :
                usual_hours_link = contract.contract_linked.usual_hours_ids
            else :
                usual_hours_link=None
           
            for usual_hour in contract.usual_hours_ids :
                """for each usual hour in the contract, the program computes its duration, the responsible
                presence hours and the time shared between the current usual hour and the corresponding one
                in the linked contract."""
                temp = self._usual_hour(usual_hour,usual_hours_link)
                duration += temp['duration']
                shared += temp['shared']
                presence += usual_hour.responsible_hours
            
            """The times are working times are balanced with the responsible hours and shared hours rates."""
            if contract.contract_linked :
                full_hours_worked = duration - (1 - float(contract.contract_linked_rate) / 100) * shared
            else :
                full_hours_worked = duration -1.0 / 3 * presence
                
            monthly_package = (float(full_hours_worked) * float(contract.worked_weeks)/ 12)
            monthly_fee = monthly_package*contract.net_rate
            res[contract.id] = {
               'monthly_fee': monthly_fee,
               'monthly_package': monthly_package,
               'hours_per_week': full_hours_worked,
               }
        return res    
    
    def onchange_linked_contract(self, cr, uid, ids, worked_weeks, total_weeks, net_rate,name, contract_linked, contract_linked_rate):
        self.modif_linked_contract(cr, uid, ids, contract_linked, contract_linked_rate)
        return self.onchange_compute(cr, uid, ids, worked_weeks, total_weeks, net_rate, name, contract_linked, contract_linked_rate)

    def onchange_linked_contract_rate(self, cr, uid, ids, worked_weeks, total_weeks, net_rate,name, contract_linked, contract_linked_rate):
        for id in ids:
            cr.execute('''update partner_contract set contract_linked_rate=100-%s where contract_linked = %s''' % (contract_linked_rate,id))
        return self.onchange_compute(cr, uid, ids, worked_weeks, total_weeks, net_rate, name, contract_linked, contract_linked_rate)
    
    """This function returns for an usual_hours A and a list of usual_hour, the duration of A and the time shared between A and the list"""
    def _usual_hour(self,usual_hour,susual_hours):
    
        begin = usual_hour.hour_from
        end = usual_hour.hour_to
        day_of_week = usual_hour.day_of_week
        shared = 0
        presence = usual_hour.responsible_hours
        duration = end-begin
        
        if susual_hours :
            for susual_hour in susual_hours :
                if susual_hour.day_of_week == day_of_week and susual_hour.hour_from <= end and susual_hour.hour_to>= begin :
                    shared += min(susual_hour.hour_to,end)-max(susual_hour.hour_from,begin)
                    presence=0
        
        return {'shared' : shared, 'duration' : duration, 'presence' : presence}
    
    def _get_period_ids(self, cr, uid, ids, field_name, arg, context={}):
        result = {}
        period_obj = self.pool.get('service.period')
        for record in self.browse(cr, uid, ids, context):
            read = self.read(cr, uid, [record.id],['partner_id'])[0]
            if read['partner_id']:
                period_ids = period_obj.search(cr, uid, [
                                                         ('partner_id.id','=',read['partner_id'][0]),
                                                         ('type','=','partner'),
                                                         ])
                result[record.id] = period_ids
            else:
                result[record.id] = []

        return result
    
    _columns = {
        'user_id': fields.many2one('res.users', 'Manager'),
        'gross_rate':fields.float('Gross rate'),
        'net_rate':fields.float('Net rate'),
        'worked_weeks':fields.integer('Number of worked weeks', size=2),
        'sale_order_id': fields.many2one('sale.order', 'Quotation'),
        'monthly': fields.boolean('Monthly'),
        'employee_id': fields.many2many('hr.employee', 'worker_contract_rel', 'contract_id', 'employee_id', 'Workers'),
        'agency' : fields.many2one('res.partner.address', 'Agency'),
        'main_contract' : fields.boolean('Main contract'),
        'period_ids' : fields.function(_get_period_ids, method=True, type='many2many', relation="service.period", string="Periods"),
    }

    _defaults = {
        'user_id': lambda obj, cr, uid, context: uid,
        'active': lambda *a: True
    }

partner_contract()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:heures effectives / semaine x Nbre semaines travaillees / an / 12
