# -*- coding: utf-8 -*-
#################################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2012 Julius Network Solutions SARL <contact@julius.fr>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#################################################################################

import datetime
from osv import fields, osv
from tools.translate import _

class period_validation(osv.osv_memory):
    _description='Period Validation'
    _name = 'period.validation'

    def get_period(self, cr, uid, context = None, employee_id = None):
        if context == None:
            return False
        period_obj = self.pool.get('service.period')
        type_validation = context.get('type_validation',False)
        today = datetime.datetime.now()
        month = today.month
        year = today.year
        if context.get('new_window',False) and not employee_id:
            period_id = False
        else:
            if type_validation == 'manager':
                if not employee_id:
                    period_id = context.get('active_ids',[])
                    if not period_id:
                        period_id = context.get('active_id',False)
                        if period_id:
                            period_id = [period_id]
                    period = period_obj.browse(cr, uid, period_id, context=context)
                    for p in period:
                        if p.type != 'employee':
                            period_id = False
                else:
                    employee = self.pool.get('hr.employee').browse(cr, uid, employee_id, context=context)
                    user = False
                    if employee:
                        user = employee.user_id.id
                    if user:
                        period_id = period_obj.search(cr, uid, [
                                                                ('employee_id.user_id.id','=',user),
                                                                ('employee_id', '<>', False),
                                                                ('month','=',month),
                                                                ('year','=',year),
                                                                ], context=context)
            elif type_validation == 'partner':
                period_id = period_obj.search(cr, uid, [
                                                       ('partner_id.partner_user_id.id','=',uid),
                                                       ('partner_id', '<>', False),
                                                       ('month','=',month),
                                                       ('year','=',year),
                                                       ], context=context)
            elif type_validation == 'employee':
                period_id = period_obj.search(cr, uid, [
                                                       ('employee_id.user_id.id','=',uid),
                                                       ('employee_id', '<>', False),
                                                       ('month','=',month),
                                                       ('year','=',year),
                                                       ], context=context)
        if period_id:
            return period_id[0]
        return False
        

    def _get_name(self, cr, uid, context=None):
        if context == None:
            return False
        name = 'no_period'
        period_obj = self.pool.get('service.period')
        service_obj = self.pool.get('service.job')
        invoice_obj = self.pool.get('account.invoice')
        period = self.get_period(cr, uid, context)
        if period:
            period_id = period_obj.read(cr, uid, [period], ['invoice_id','state','employee_service_ids','partner_service_ids'], context=context)[0]
            if period_id['employee_service_ids']:
                service_ids = service_obj.read(cr, uid, period_id['employee_service_ids'], ['invoiced'], context=context)
                for service in service_ids:
                    if service['invoiced']:
                        name = 'invoiced'
                        break
            elif  period_id['partner_service_ids']:
                service_ids = service_obj.read(cr, uid, period_id['partner_service_ids'], ['invoiced'], context=context)
                for service in service_ids:
                    if service['invoiced']:
                        name = 'invoiced'
                        break
            elif period_id['invoice_id']:
                invoice = invoice_obj.read(cr, uid, [period_id['invoice_id'][0]], ['state'], context=context)[0]
                if invoice['state'] not in ['draft','cancel'] :
                    name = 'invoiced'
                else:
                    name = 'not_invoiced'
            elif period_id['state'] == 'draft':
                name = 'not_validated'
            else:
                name = 'not_invoiced'
        return name
    
    def _get_type(self, cr, uid, context=None):
        if context == None:
            return False
        type_validation = context.get('type_validation',False)
        return type_validation
    
    def _get_parnter(self, cr, uid, context=None):
        if context == None:
            return False
        type_validation = context.get('type_validation',False)
        if type_validation == 'partner':
            partner = self.pool.get('res.partner').search(cr, uid, [
                                                                   ('partner_user_id.id','=',uid),
                                                                   ('partner_user_id','<>', False)
                                                                   ], context=context)
            if partner:
                return partner[0]
        return False
    
    def _get_employee(self, cr, uid, context=None):
        if context == None:
            return False
        type_validation = context.get('type_validation',False)
        employee = False
        if type_validation == 'manager':
            period = self.get_period(cr, uid, context)
            if period:
                employee = self.pool.get('service.period').read(cr, uid, period, ['employee_id'], context=context)['employee_id']
        elif type_validation == 'employee':
            employee = self.pool.get('hr.employee').search(cr, uid, [
                                                                   ('user_id.id','=',uid),
                                                                   ('user_id','<>', False)], context=context)
        if employee:
            return employee[0]
        return False
    
    def _get_period(self, cr, uid, context=None):
        if context == None:
            return False
        period = self.get_period(cr, uid, context)
        if period:
            return period
        return False
    
    _columns = {
        'name': fields.selection([
            ('no_period','You\'ve got any period to validate'),
            ('invoiced','The period has already been invoiced'),
            ('not_validated','Your period is not validated'),
            ('not_invoiced', 'Your period is already validated but not invoiced yet')
        ], 'Name', readonly=True),
        'partner_id': fields.many2one('res.partner', 'Partner', readonly=True),
        'employee_id': fields.many2one('hr.employee', 'Employee'),
        'type' : fields.selection([
            ('partner','Partner'),
            ('employee','Employee'),
            ('manager','Manager')
        ], 'type', readonly=True),
        'period_id' : fields.many2one('service.period', 'Period', required=True),
        'service_ids': fields.many2many('service.job', 'service_wizard_rel', 'wiz_id', 'service_id', 'Periods\' service jobs'),
    }
    
    _defaults = {
        'type': _get_type,
        'partner_id': _get_parnter,
        'employee_id': _get_employee,
        'period_id': _get_period,
    }
    
    def action_close(self,cr,uid,ids,conect=None):
        return {'type': 'ir.actions.act_window_close'}

    def action_validated(self, cr, uid, ids, context=None):
        period_id = self.browse(cr, uid, ids[0], context=context).period_id.id
        period_obj = self.pool.get('service.period')
        period_obj._get_validated_hours(cr, uid, [period_id], context=context)
        period_obj._validated_period(cr, uid, [period_id], context=context)
        return  {'type': 'ir.actions.act_window_close'}
    
    def on_change_employee(self, cr, uid, id, employee_id, context):
        res = {'value' : {
            'employee_id' : False,
        }}
        if not employee_id:
            res['value']['period_id'] = False
            res['value']['name'] = 'no_period'
            res['value']['service_ids'] = False
            return res
        else :
            period_id = self.get_period(cr, uid, context, employee_id)
            res['value']['employee_id'] = employee_id
            res['value']['period_id'] = period_id or False
            service_obj = self.pool.get('service.job')
            service_ids = False
            if period_id:
                service_ids = service_obj.search(cr, uid, [
                    ('employee_period_id','=',period_id),
                ], context=context)
                name = False
                period_obj = self.pool.get('service.period')
                invoice_obj = self.pool.get('account.invoice')
                period_id = period_obj.read(cr, uid, [period_id], ['invoice_id','state','employee_service_ids','partner_service_ids'], context=context)[0]
                if period_id['employee_service_ids']:
                    service_ids = service_obj.read(cr, uid, period_id['employee_service_ids'], ['invoiced'], context=context)
                    for service in service_ids:
                        if service['invoiced']:
                            name = 'invoiced'
                            break
                elif  period_id['partner_service_ids']:
                    service_ids = service_obj.read(cr, uid, period_id['partner_service_ids'], ['invoiced'], context=context)
                    for service in service_ids:
                        if service['invoiced']:
                            name = 'invoiced'
                            break
                elif period_id['invoice_id']:
                    invoice = invoice_obj.read(cr, uid, [period_id['invoice_id'][0]], ['state'], context=context)[0]
                    if invoice['state'] not in ['draft','cancel'] :
                        name = 'invoiced'
                    else:
                        name = 'not_invoiced'
                elif period_id['state'] == 'draft':
                    name = 'not_validated'
                else:
                    name = 'not_invoiced'
            res['value']['service_ids'] = service_ids
            if name:
                res['value']['name'] = name
        return res
    
    def on_change_period(self, cr, uid, id, period_id, type = None):
        res = {'value' : {
            'period_id' : False,
            'name' : 'no_period',
            'childcare_ids' : False,
        }}
        if not period_id:
            return res
        else :
            res['value']['period_id'] = period_id
            service_obj = self.pool.get('service.job')
            period_obj = self.pool.get('service.period')
            invoice_obj = self.pool.get('account.invoice')
            if type == 'partner':
                service_ids = service_obj.search(cr, uid, [
                                                           ('partner_period','=',period_id),
                                                           ], context=context)
            else:
                service_ids = service_obj.search(cr, uid, [
                                                           ('employee_period','=',period_id),
                                                           ], context=context)
            res['value']['service_ids'] = service_ids
            name = False
            period_id = period_obj.read(cr, uid, [period_id], ['invoice_id','state','employee_service_ids','partner_service_ids'], context=context)[0]
            if period_id['employee_service_ids']:
                service_ids = service_obj.read(cr, uid, period_id['employee_service_ids'], ['invoiced'], context=context)
                for service in service_ids:
                    if service['invoiced']:
                        name = 'invoiced'
                        break
            elif  period_id['partner_service_ids']:
                service_ids = service_obj.read(cr, uid, period_id['partner_service_ids'], ['invoiced'], context=context)
                for service in service_ids:
                    if service['invoiced']:
                        name = 'invoiced'
                        break
            elif period_id['invoice_id']:
                invoice = invoice_obj.read(cr, uid, [period_id['invoice_id'][0]], ['state'], context=context)[0]
                if invoice['state'] not in ['draft','cancel'] :
                    name = 'invoiced'
                else:
                    name = 'not_invoiced'
            elif period_id['state'] == 'draft':
                name = 'not_validated'
            else:
                name = 'not_invoiced'
            if name:
                res['value']['name'] = name
        return res

period_validation()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
