# -*- coding: utf-8 -*-
#################################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2012 Julius Network Solutions SARL <contact@julius.fr>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#################################################################################

from osv import fields, osv
from tools.translate import _

class service_state(osv.osv):
    _inherit = 'service.state'
    _description = 'Service state'
    _columns = {
        'to_invoice_employee': fields.many2one('hr_timesheet_invoice.factor', 'Invoice on Timesheet & Costs Employee value'),
    }
service_state()

class account_analytic_line(osv.osv):
    _inherit = 'account.analytic.line'
    _columns = {
        'to_invoice_employee': fields.many2one('hr_timesheet_invoice.factor', 'Invoice on Timesheet & Costs Employee value'),
        'payslip_id': fields.many2one('hr.payslip', 'Payslip'),
    }
    
    def copy(self, cr, uid, id, default=None, context=None):
        if not default:
            default = {}
        default['payslip_id'] = False
        default['to_invoice_employee'] = False
        return super(account_analytic_line, self).copy(cr, uid, id, default=default, context=context)
    
account_analytic_line()

class hr_timesheet_invoice_factor(osv.osv):
    _inherit = 'hr_timesheet_invoice.factor'
    _columns = {
        'payslip_code': fields.char('Workdays code', size=16),
        'input_code': fields.char('Input code', size=16),
        'employee_name': fields.char('Payslip Name', size=128, translate=True, help="Label for the payslip line"),
    }
hr_timesheet_invoice_factor()

class hr_analytic_timesheet(osv.osv):
    _inherit = 'hr.analytic.timesheet'
    
    def get_timesheet_vals(self, cr, uid, id, context=None):
        timesheet_vals = super(hr_analytic_timesheet, self).get_timesheet_vals(cr, uid, id, context=context)
        timesheet = self.browse(cr, uid, id, context=context)
        timesheet_vals.update({'to_invoice_employee': timesheet.state_id.to_invoice_employee and timesheet.state_id.to_invoice_employee.id or False})
        return timesheet_vals

hr_analytic_timesheet()

class hr_timesheet_sheet(osv.osv):
    _inherit = "hr_timesheet_sheet.sheet"
    
    def payslip_ids(self, cr, uid, ids, field_name, arg, context={}):
        result = {}
        for record in self.browse(cr, uid, ids, context):
            result[record.id] = []
            for x in record.employee_line_ids:
                if x.payslip_id and x.payslip_id.id not in result[record.id]:
                    result[record.id].append(x.payslip_id.id)
        return result

    _columns = {
        'payslip_ids' : fields.function(payslip_ids, method=True, type='many2many', relation="hr.payslip", string="Related Payslip"),
    }
    
    def create_payslip(self, cr, uid, ids, context=None):
        return True

hr_timesheet_sheet()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
