# -*- coding: utf-8 -*-
#################################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2012 Julius Network Solutions SARL <contact@julius.fr>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#################################################################################

import tools
import time
import datetime
from mx import DateTime
from tools.translate import _
from osv import osv, fields

def format_date_tz(date, tz):
    dt_value = datetime.datetime.strptime(date, "%Y-%m-%d %H:%M:%S")
    import pytz
    src_tz = pytz.timezone(tz)
    dst_tz = pytz.timezone('UTC')
    src_dt = src_tz.localize(dt_value, is_dst=True)
    dt_value = src_dt.astimezone(dst_tz)
    return dt_value

class service_fill(osv.osv_memory):
    _name = 'service.fill'
    _description = 'Service Fill'
    
    _columns = {
        'date_start': fields.date("Start Date", required=True),
        'date_end': fields.date("End Date", required=True),
    }
    
    def test_date(self, cr, uid, date, usual, context=None):
        if (usual.date_from and date >= datetime.datetime(*(time.strptime(usual.date_from, '%Y-%m-%d')[0:6])) or not usual.date_from and True) \
            and (usual.contract_id.date_start and date >= datetime.datetime(*(time.strptime(usual.contract_id.date_start, '%Y-%m-%d')[0:6])) or True) \
            and (usual.contract_id.date and date <= datetime.datetime(*(time.strptime(usual.contract_id.date, '%Y-%m-%d')[0:6])) or not usual.contract_id.date and True) \
            and (usual.employee_contract_id.date_start and date >= datetime.datetime(*(time.strptime(usual.employee_contract_id.date_start, '%Y-%m-%d')[0:6])) or True) \
            and (usual.employee_contract_id.date_end and date <= datetime.datetime(*(time.strptime(usual.employee_contract_id.date_end, '%Y-%m-%d')[0:6])) or not usual.employee_contract_id.date_end and True) \
            and (usual.date_from and date >= datetime.datetime(*(time.strptime(usual.date_from, '%Y-%m-%d')[0:6])) or not usual.date_from and True):
            return True
        return False
    
    def get_vals(self, cr, uid, date, usual, context=None):
        if context is None:
            context = {}
        new_date_start = datetime.datetime.strftime(date + datetime.timedelta(hours=int(usual.hour_from or 0),minutes=round(((usual.hour_from or 0)-int(usual.hour_from or 0))*60)), '%Y-%m-%d %H:%M:00')
        new_date_end = datetime.datetime.strftime(date + datetime.timedelta(hours=int(usual.hour_to or 0),minutes=round(((usual.hour_to or 0)-int(usual.hour_to or 0))*60)), '%Y-%m-%d %H:%M:00')
        
        employee_contract = usual.employee_contract_id
        employee = employee_contract.employee_id
        contract = usual.contract_id
        journal_id = employee and employee.journal_id and employee.journal_id.id or False
        prod_id = employee and employee.product_id and employee.product_id.id or False
        duration = (DateTime.strptime(new_date_end,'%Y-%m-%d %H:%M:00') - DateTime.strptime(new_date_start,'%Y-%m-%d %H:%M:00'))
        unit_amount = duration.hours or 0
        
        new_date_start = format_date_tz(new_date_start, context.get('tz', 'UTC'))
        new_date_end = format_date_tz(new_date_end, context.get('tz', 'UTC'))
        vals = {
            'employee_contract_id': employee_contract and employee_contract.id or False,
            'user_id': employee and employee.user_id.id or False,
            'product_id': prod_id,
            'journal_id': journal_id,
            'contract_id': contract and contract.id or False,
            'date_start': new_date_start,
            'date_end': new_date_end,
            'usual_date_start': new_date_start,
            'usual_date_end': new_date_end,
            'date': datetime.datetime.strftime(date,'%Y-%m-%d'),
            'unit_amount': unit_amount,
            'usual_state_id': usual.state_id and usual.state_id.id or False,
            'state_id': usual.state_id and usual.state_id.id or False,
            'account_id': contract.analytic_account_id and contract.analytic_account_id.id or False,
            'name': employee.name + (contract.partner_id and ' - ' + contract.partner_id.name or ''),
            'employee_id': employee.id,
        }
        return vals
    
    def create_hr_timesheet(self, cr, uid, date, usual, context=None):
        if context==None:
            context={}
        new_date_start = datetime.datetime.strftime(date + datetime.timedelta(hours=int(usual.hour_from or 0),minutes=((usual.hour_from or 0)-int(usual.hour_from or 0))*60), '%Y-%m-%d %H:%M:00')
        new_date_end = datetime.datetime.strftime(date + datetime.timedelta(hours=int(usual.hour_to or 0),minutes=((usual.hour_to or 0)-int(usual.hour_to or 0))*60), '%Y-%m-%d %H:%M:00')
        
        timesheet_obj = self.pool.get('hr.analytic.timesheet')
        employee_contract = usual.employee_contract_id
        employee = employee_contract.employee_id
        contract = usual.contract_id
        journal_id = employee and employee.journal_id and employee.journal_id.id or False
        prod_id = employee and employee.product_id and employee.product_id.id or False
        duration = (DateTime.strptime(new_date_end,'%Y-%m-%d %H:%M:00') - DateTime.strptime(new_date_start,'%Y-%m-%d %H:%M:00'))
        unit_amount = duration.hours or 0
        
        vals = self.get_vals(cr, uid, date, usual, context=context)
        res = timesheet_obj.on_change_unit_amount2(cr, uid, id=False, prod_id=prod_id, unit_amount=unit_amount, company_id=False, unit=False, journal_id=journal_id, employee_contract_id=employee_contract and employee_contract.id or False, context=context)
        if res.get('value'):
            if res['value'].get('amount'):
                vals.update({'amount': res['value'].get('amount', 0)})
            if res['value'].get('product_uom_id'):
                vals.update({'product_uom_id': res['value'].get('product_uom_id', False)})
            if res['value'].get('general_account_id'):
                vals.update({'general_account_id': res['value'].get('general_account_id', False)})
        return timesheet_obj.create(cr, uid, vals, context=context)
        
    def _get_usual_hours(self, cr, uid, contract, context=None):
        return contract.working_hours and contract.working_hours.attendance_ids or False
    
    def fill_service(self, cr, uid, ids, context=None):
        if context==None:
            context={}
        hr_contract_obj = self.pool.get('hr.contract')
        contract_obj = self.pool.get('contract.contract')
#        timesheet_obj = self.pool.get('hr.analytic.timesheet')
#        sheet_obj = self.pool.get('hr_timesheet_sheet.sheet')
        
        current_data = self.browse(cr, uid, ids, context)
        
        for data in current_data:
            date_start = data.date_start
            date_end = data.date_end
            
            if date_end < date_start:
                raise osv.except_osv(_('Error !'), _('Please choose an end date greater than the start date'))
            
            contract_ids = hr_contract_obj.search(cr, uid, [
                                    ('date_start', '<=', date_end),
                                    '|',('date_end', '>=', date_start),
                                    ('date_end', '=', False),
                                    ], context=context)
            contract_data = hr_contract_obj.browse(cr, uid, contract_ids, context=context)
            for contract in contract_data:
                date = datetime.datetime(*(time.strptime(date_start, '%Y-%m-%d')[0:6]))
                end = datetime.datetime(*(time.strptime(date_end, '%Y-%m-%d')[0:6]))
                usual_hours = self._get_usual_hours(cr, uid, contract, context=context)
                if usual_hours:
                    while date <= end:
                        day = date.weekday()
                        day_usual = [x for x in usual_hours if int(x.dayofweek) == day]
                        for usual in day_usual:
                            if usual.contract_id and usual.contract_id.active and usual.employee_contract_id and usual.employee_contract_id.active:
                                if self.test_date(cr, uid, date, usual, context=context):
                                    self.create_hr_timesheet(cr, uid, date, usual, context=context)
                        date += datetime.timedelta(days=1)
        return {'type': 'ir.actions.act_window_close'}
service_fill()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

