# -*- coding: utf-8 -*-
#################################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2012 Julius Network Solutions SARL <contact@julius.fr>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#################################################################################


import time
import datetime
from mx import DateTime
from osv import fields,osv
from tools.translate import _

class one2many_mod(fields.one2many):
    def get(self, cr, obj, ids, name, user=None, offset=0, context=None, values=None):
        if context is None:
            context = {}

        if values is None:
            values = {}

        res5 = obj.read(cr, user, ids, ['date_current'], context=context)
        res6 = {}
        for r in res5:
            res6[r['id']] = r['date_current']

        ids2 = []
        for id in ids:
            dom = []
            if id in res6:
                dom = [('date', '=', res6[id]), ('partner_sheet_id', '=', id)]
            ids2.extend(obj.pool.get(self._obj).search(cr, user,
                dom, limit=self._limit))
        res = {}
        for i in ids:
            res[i] = []
        for r in obj.pool.get(self._obj)._read_flat(cr, user, ids2,
                [self._fields_id], context=context, load='_classic_read'):
            if r[self._fields_id]:
                res[r[self._fields_id][0]].append(r['id'])

        return res

class hr_analytic_timesheet(osv.osv):
    _inherit = 'hr.analytic.timesheet'
    _order = 'sheet_id, date_start, partner_sheet_id'
    
    def get_create_partner_sheet(self, cr, uid, date, partner_id, context=None):
        sheet_obj = self.pool.get('hr_timesheet_sheet.sheet')
        sheet_id = False
        sheet_ids = sheet_obj.search(cr, uid,
            [('date_to', '>=', date), ('date_from', '<=', date),
                 ('partner_id', '=', partner_id)],
                context=context)
        if not sheet_ids:
            end = self._period(cr, uid, date, field_name='end', invoice_day=21, context=context)
            begin = self._period(cr, uid, date, field_name='begin', invoice_day=21, context=context)
            name = datetime.datetime.strftime(end,'%m/%Y')
            vals = {'type': 'partner', 'partner_id': partner_id, 'date_from': begin, 'date_to': end, 'date_current': begin, 'name': name}
            sheet_id = sheet_obj.create(cr, uid, vals, context=context)
        return sheet_ids and sheet_ids[0] or sheet_id

    def _partner_sheet(self, cursor, user, ids, name, args, context=None):
        sheet_obj = self.pool.get('hr_timesheet_sheet.sheet')
        res = {}.fromkeys(ids, False)
        for ts_line in self.browse(cursor, user, ids, context=context):
            if ts_line.account_id and ts_line.account_id.partner_id:
                sheet_ids = sheet_obj.search(cursor, user,
                    [('date_to', '>=', ts_line.date), ('date_from', '<=', ts_line.date),
                     ('partner_id', '=', ts_line.account_id.partner_id.id)],
                    context=context)
                if sheet_ids:
                # [0] because only one sheet possible for an employee between 2 dates
                    res[ts_line.id] = sheet_obj.name_get(cursor, user, sheet_ids, context=context)[0]
        return res

    def _get_hr_timesheet_partner_sheet(self, cr, uid, ids, context=None):
        ts_line_ids = []
        for ts in self.browse(cr, uid, ids, context=context):
            if ts.partner_id:
                cr.execute("""
                        SELECT l.id
                            FROM hr_analytic_timesheet l
                        INNER JOIN account_analytic_line al
                            ON (l.line_id = al.id)
                        WHERE %(date_to)s >= al.date
                            AND %(date_from)s <= al.date
                            AND al.account_id IN (SELECT id FROM account_analytic_account WHERE partner_id = %(partner_id)s )
                        GROUP BY l.id""", {'date_from': ts.date_from,
                                            'date_to': ts.date_to,
                                            'partner_id': ts.partner_id.id,})
                ts_line_ids.extend([row[0] for row in cr.fetchall()])
        return ts_line_ids
    
    def _get_account_analytic_line(self, cr, uid, ids, context=None):
        ts_line_ids = self.pool.get('hr.analytic.timesheet').search(cr, uid, [('line_id', 'in', ids)])
        return ts_line_ids

    _columns = {
        'partner_sheet_id': fields.function(_partner_sheet, string='Partner Sheet',
            type='many2one', relation='hr_timesheet_sheet.sheet',
            store = {
                'hr_timesheet_sheet.sheet': (_get_hr_timesheet_partner_sheet, ['partner_id', 'date_from', 'date_to'], 10),
                'account.analytic.line': (_get_account_analytic_line, ['account_id', 'date'], 10),
                'hr.analytic.timesheet': (lambda self,cr,uid,ids,context=None: ids, ['line_id', 'date_start', 'date_end'], 10),
            },
        ),
    }
    
    def create(self, cr, uid, vals, context=None):
        if context == None:
            context = {}
        date_end = vals.get('date_end')
        date_start = vals.get('date_start')
        if date_start or date_end:
            vals['date'] = date_start or date_end or vals.get('date')
        if vals.get('date') and vals.get('account_id'):
            account = self.pool.get('account.analytic.account').browse(cr, uid, vals.get('account_id'), context=context)
            if account and account.partner_id:
                vals['partner_sheet_id'] = self.get_create_partner_sheet(cr, uid, vals.get('date'), account.partner_id.id, context=context)
        return super(hr_analytic_timesheet, self).create(cr, uid, vals, context=context)
    
hr_analytic_timesheet()

class hr_timesheet_sheet(osv.osv):
    _inherit = "hr_timesheet_sheet.sheet"

    _columns = {
        'type': fields.selection([('employee','Employee'),('partner','Partner')], 'Type of sheet', required=True),
        'employee_id': fields.many2one('hr.employee', 'Employee'),
        'partner_id': fields.many2one('res.partner', 'Partner'),
        'partner_line_ids': fields.one2many('hr.analytic.timesheet', 'partner_sheet_id', 'Lines', readonly=False),
        'partner_timesheet_ids' : one2many_mod('hr.analytic.timesheet', 'partner_sheet_id',
            'Timesheet lines', domain=[('date', '=', time.strftime('%Y-%m-%d'))],
            readonly=True, states={
                'draft': [('readonly', False)],
                'new': [('readonly', False)]}
            ),
    }
    
    _defaults = {
        'type': lambda *a: 'employee',
    }
    
    def _sheet_date(self, cr, uid, ids, forced_user_id=False, context=None):
        for sheet in self.browse(cr, uid, ids, context=context):
            new_user_id = forced_user_id or sheet.user_id and sheet.user_id.id
            partner_id = sheet.partner_id and sheet.partner_id.id or False
            if new_user_id:
                cr.execute('SELECT id \
                    FROM hr_timesheet_sheet_sheet \
                    WHERE (date_from <= %s and %s <= date_to) \
                        AND type=\'employee\' \
                        AND user_id=%s \
                        AND id <> %s',(sheet.date_to, sheet.date_from, new_user_id, sheet.id))
                if cr.fetchall():
                    return False
            elif partner_id:
                cr.execute('SELECT id \
                    FROM hr_timesheet_sheet_sheet \
                    WHERE (date_from <= %s and %s <= date_to) \
                        AND type=\'partner\' \
                        AND partner_id=%s \
                        AND id <> %s',(sheet.date_to, sheet.date_from, partner_id, sheet.id))
                if cr.fetchall():
                    return False
        return True
    
    _constraints = [
        (_sheet_date, 'You cannot have 2 timesheets that overlaps !\nPlease use the menu \'My Current Timesheet\' to avoid this problem.', ['date_from','date_to']),
    ]

hr_timesheet_sheet()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
