# -*- encoding: utf-8 -*-
#################################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2012 Julius Network Solutions SARL <contact@julius.fr>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#################################################################################

from osv import osv, fields
from tools.translate import _

class service_state_change(osv.osv_memory):
    _name = 'service.state.change'
    
    def _get_employee(self, cr, uid, active_id=None, context=None):
        res = False
        if active_id:
            active = self.pool.get('hr.analytic.timesheet').browse(cr, uid, active_id, context=context).employee_contract_id
            res = active and active.employee_id and active.employee_id.id or False
        return res
    
    def _get_employee_contract(self, cr, uid, active_id=None, context=None):
        res = False
        if active_id:
            active = self.pool.get('hr.analytic.timesheet').browse(cr, uid, active_id, context=context).employee_contract_id
            res = active and active.id or False
        return res
    
    def _get_contract(self, cr, uid, active_id=None, context=None):
        res = False
        if active_id:
            active = self.pool.get('hr.analytic.timesheet').browse(cr, uid, active_id, context=context).contract_id
            res = active and active.id or False
        return res
    
    def _get_date(self, cr, uid, active_id=None, context=None):
        res = False
        if active_id:
            res = self.pool.get('hr.analytic.timesheet').browse(cr, uid, active_id, context=context).date_start
        return res
    
    def _get_state(self, cr, uid, active_id=None, context=None):
        res = False
        if active_id:
            active = self.pool.get('hr.analytic.timesheet').browse(cr, uid, active_id, context=context).state_id
            res = active and active.id or False
        return res
    
    _columns = {
        'date_start': fields.date('Date start', required=True),
        'date_end': fields.date('Date end', required=True),
        'state_id': fields.many2one('service.state', 'State', required=True),
        'employee_id': fields.many2one('hr.employee', 'Employee', required=True),
        'employee_contract_id': fields.many2one('hr.contract', 'Work Contract'),
        'contract_id': fields.many2one('contract.contract', 'Partner Contract'),
    }
    
    _defaults = {
        'employee_id': lambda self, cr, uid, context: self._get_employee(cr, uid, context.get('active_id', False), context),
        'employee_contract_id': lambda self, cr, uid, context: self._get_employee_contract(cr, uid, context.get('active_id', False), context),
        'contract_id': lambda self, cr, uid, context: self._get_contract(cr, uid, context.get('active_id', False), context),
        'state_id': lambda self, cr, uid, context: self._get_state(cr, uid, context.get('active_id', False), context),
        'date_start': lambda self, cr, uid, context: self._get_date(cr, uid, context.get('active_id', False), context),
        'date_end': lambda self, cr, uid, context: self._get_date(cr, uid, context.get('active_id', False), context),
    }
    
    def change_state(self, cr, uid, ids, context=None):
        if context == None:
            context = {}
        timesheet_obj = self.pool.get('hr.analytic.timesheet')
        for current in self.browse(cr, uid, ids, context=context):
            domain = []
            employee_contract_id = current.employee_contract_id and current.employee_contract_id.id or False
            if employee_contract_id:
                domain.append(('employee_contract_id', '=', employee_contract_id))
            contract_id = current.contract_id and current.contract_id.id or False
            if contract_id:
                domain.append(('contract_id', '=', contract_id))
            if not employee_contract_id:
                user_id = current.employee_id and current.employee_id.user_id and current.employee_id.user_id.id or False
                if user_id:
                    domain.append(('user_id', '=', user_id))
            date_start = current.date_start
            if date_start:
                domain.append(('date_start', '>=', date_start))
            date_end = current.date_end
            if date_end:
                domain.append(('date_end', '<=', date_end))
            
            service_ids = timesheet_obj.search(cr, uid, domain, context=context)
            state_id = current.state_id and current.state_id.id or False
            if state_id and service_ids:
                timesheet_obj.write(cr, uid, service_ids, {'state_id': state_id}, context)
        return {'type': 'ir.actions.act_window_close'}
            
service_state_change()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
